// http://www.amin-ahmadi.com

#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    void *temp;

    if(WMCreateProfileManager(&pProfMan) == S_OK)
    {
        if(pProfMan->QueryInterface(IID_IWMCodecInfo3, &temp) == S_OK)
        {
            pCodecInfo = static_cast<IWMCodecInfo3*>(temp);
            if( GetCodecNames() != S_OK )
            {
                QMessageBox::critical(this, "Error", "Something went wrong. Check this error msg in source code!");
            }
        }
        else
        {
            QMessageBox::critical(this, "Error", "Something went wrong. Check this error msg in source code!");
        }

    }
    else
    {
        QMessageBox::critical(this, "Error", "Something went wrong. Check this error msg in source code!");
    }

}

MainWindow::~MainWindow()
{
    delete ui;
}

HRESULT MainWindow::GetCodecNames()
{
    HRESULT hr = S_OK;
    DWORD dwCodecIndex = 0;
    DWORD   cCodecs  = 0;
    WCHAR*  pwszCodecName  = NULL;
    DWORD   cchCodecName     = 0;

    // Retrieve the number of supported audio codecs on the system.
    hr = pCodecInfo->GetCodecInfoCount(WMMEDIATYPE_Audio, &cCodecs);

    if(SUCCEEDED(hr))
        QMessageBox::information(this, "Info", "Number of audio codecs: " + QString::number(cCodecs));
    else
    {
        QMessageBox::critical(this, "Error", "Could not get the count of audio codecs.");
        return hr;
    }

    // Loop through all the audio codecs.
    for(dwCodecIndex = 0; dwCodecIndex < cCodecs; dwCodecIndex++)
    {
        // Get the codec name:
        // First, get the size of the name.
        hr = pCodecInfo->GetCodecName(WMMEDIATYPE_Audio,
                                      dwCodecIndex,
                                      NULL,
                                      &cchCodecName);
        if(FAILED(hr))
        {
            QMessageBox::critical(this, "Error", "Could not get the size of the codec name.");
            return hr;
        }

        // Allocate a string of the appropriate size.
        pwszCodecName = new WCHAR[cchCodecName];
        if(pwszCodecName == NULL)
        {
            QMessageBox::critical(this, "Error", "Could not allocate memory.");
            return E_OUTOFMEMORY;
        }

        // Retrieve the codec name.
        hr = pCodecInfo->GetCodecName(WMMEDIATYPE_Audio,
                                      dwCodecIndex,
                                      pwszCodecName,
                                      &cchCodecName);
        if(FAILED(hr))
        {
            delete[] pwszCodecName;
            QMessageBox::critical(this, "Error", "Could not get the codec name.");
            return hr;
        }

        // Print the codec name.
        ui->audio_list->addItem(QString::number(dwCodecIndex) + "  " + QString::fromWCharArray(pwszCodecName));

        // Clean up for the next iteration.
        delete[] pwszCodecName;
        pwszCodecName = NULL;
        cchCodecName  = 0;
    }

    // Retrieve the number of supported video codecs on the system.
    hr = pCodecInfo->GetCodecInfoCount(WMMEDIATYPE_Video, &cCodecs);

    if(SUCCEEDED(hr))
        QMessageBox::information(this, "Info", "Number of video codecs: " + QString::number(cCodecs));
    else
    {
        QMessageBox::critical(this, "Error", "Could not get the count of video codecs.");
        return hr;
    }

    // Loop through all the video codecs.
    for(dwCodecIndex = 0; dwCodecIndex < cCodecs; dwCodecIndex++)
    {
        // Get the codec name:
        // First, get the size of the name.
        hr = pCodecInfo->GetCodecName(WMMEDIATYPE_Video,
                                      dwCodecIndex,
                                      NULL,
                                      &cchCodecName);
        if(FAILED(hr))
        {
            QMessageBox::critical(this, "Error", "Could not get the size of the codec name.");
            return hr;
        }

        // Allocate a string of the appropriate size.
        pwszCodecName = new WCHAR[cchCodecName];
        if(pwszCodecName == NULL)
        {
            QMessageBox::critical(this, "Error", "Could not allocate memory.");
            return E_OUTOFMEMORY;
        }

        // Retrieve the codec name.
        hr = pCodecInfo->GetCodecName(WMMEDIATYPE_Video,
                                      dwCodecIndex,
                                      pwszCodecName,
                                      &cchCodecName);
        if(FAILED(hr))
        {
            QMessageBox::critical(this, "Error", "Could not get the codec name.");
            return hr;
        }

        // Print the codec name.
        ui->video_list->addItem(QString::number(dwCodecIndex) + "  " + QString::fromWCharArray(pwszCodecName));

        delete[] pwszCodecName;
        pwszCodecName = NULL;
        cchCodecName  = 0;
    }

    return S_OK;
}
